/* Defines common to all versions of NSS */

#define NSPR20 1
#define MP_API_COMPATIBLE 1
#define NSS_3_4_CODE 1 /* Remove this when we start building NSS 4.0 by default */
#define SHLIB_SUFFIX "shlb"
